import $ from 'jquery';
import {executeFunctionByName, findGenericFunctionByName, urlDecode} from 'aitom-helpers';
window.jQuery = $;
require('nette.ajax.js');

/**
 * AitomAjax - autoinstance singleton facade for nette.ajax.js library
 *
 * @author Michal Kopecký
 */
class _AitomAjax
{

    constructor()
    {
        this._registredExtensions = [];
        this._beforeSnippetUpdateCallbacks = [];

        // init nette ajax
        if (php.ajax && $.nette) {
            // init aitom5 custom ajax actions
            this.initAjaxActions();

            // disable unique extension - it does something different than we want to
            // tohle vyplne unique rozšíření, které dělá něco jiného, než my potřebujeme
            $.nette.ext('unique', null);

            // this changes default ajaxification to use other binding method (distributed), in order to make room for custom events to work (and cancel the ajax action if needed) - partially for bc reasons
            // tohle změní způsob ajaxizace na jiný způsob bindování (distributed), aby bylo možné jiným bindem zrušit ajaxovou událost - částečně z důvodu zpětné kompatability
            var defaultSelectors = $.nette.ext('init');

            // může se iniciovat až po DOM ready
            $(() => {
                $.nette.init((ajaxHandler) => {
                    $('body').off('click', defaultSelectors.linkSelector, ajaxHandler).on('click', defaultSelectors.linkSelector, ajaxHandler);
                    $('body').off('submit', defaultSelectors.formSelector, ajaxHandler).on('submit', defaultSelectors.formSelector, ajaxHandler);
                    $(defaultSelectors.formSelector).off('click', ':image', ajaxHandler).on('click', ':image', ajaxHandler);
                    $(defaultSelectors.formSelector).off('click', ':submit', ajaxHandler).on('click', ':submit', ajaxHandler);
                    $('body').off('click', defaultSelectors.buttonSelector, ajaxHandler).on('click', defaultSelectors.buttonSelector, ajaxHandler);
                });
            });
        }
    }

    ajax(obj)
    {
        $.nette.ajax(obj);
    }

    /**
     * Add nette ajax extension
     *
     * @param name string
     * @param obj object
     */
    ext(name, obj)
    {
        $.nette.ext(name, obj);
    }

    /**
     * Add nette ajax extension with autocreate unique name
     *
     * @param obj object
     */
    addExt(obj)
    {
        const name = `aitom-extension-${this._registredExtensions.length}`;
        this._registredExtensions.push(name);
        $.nette.ext(name, obj);
    }

    /**
     * Add callback which is run right before snippet html is updated.
     *
     * @param callback string | function
     */
    addBeforeSnippetUpdateCallback(callback)
    {
        if (typeof callback === 'string') {
            this._beforeSnippetUpdateCallbacks.push(findGenericFunctionByName(callback));
        } else if (typeof callback === 'function') {
            this._beforeSnippetUpdateCallbacks.push(callback);
        } else {
            throw new Error("addBeforeSnippetUpdateCallback: unsupported callback type");
        }
    }

    /**
     * Inits and defines custom ajax mechanisms in Aitom 5
     */
    initAjaxActions()
    {
        const self = this;
        $.nette.ext('aitom-ajax', {
            init: function () {
                this.ext('snippets', true).before(function ($el) {
                    // call all registered 'before snippet update' callbacks
                    $.each(self._beforeSnippetUpdateCallbacks, function (key, callback) {
                        callback.apply(null, [$el]);
                    });
                });
            },
            before: function (xhr, settings) {

                // CALLBACK PARSING
                if (settings.nette && settings.nette.el.data('callback')) {
                    this.aitomCallback = findGenericFunctionByName(settings.nette.el.data('callback'));
                }

                // on ajax event - used in grid only - remove after grid is replaced
                // @todo APET-1111
                if (settings.nette) {
                    var origUrl = urlDecode(settings.url);
                    var params = {
                        url: origUrl,
                        data: {},
                        "callback": this.aitomCallback,
                        cancelRequest: false
                    };
                    if (settings.nette.el.data().onajax) {
                        var onAjax = settings.nette.el.data().onajax;
                        if (typeof onAjax === 'string') {
                            executeFunctionByName(onAjax, [settings.nette.el[0], params], this);
                        } else if (typeof onAjax === 'function') {
                            onAjax.apply(this, [settings.nette.el[0], params]);
                        }
                    }
                    if (origUrl !== params.url) {
                        settings.url = params.url;
                    }
                    if (params.cancelRequest == true) {
                        return false;
                    }
                }

            },
            start: function (xhr, settings) {

                if (this.currentXhr !== null && !settings.disableAbort) {
                    // abort new requests until old one is complete
                    xhr.abort();
                }
                this.currentXhr = xhr;
            },
            success: function (payload) {

                // call specific callbacks by string name
                // @todo APET-1111 pravdepodobne bude mozne odstranit po odstraneni podpory $this->payload->callback[]
                if (payload.callback && payload.callback.length > 0) {
                    var calledCallback = []; // call only once!
                    $.each(payload.callback, function (i, callback) {
                        if ($.inArray(callback, calledCallback) != -1) {
                            return; // continue
                        }
                        executeFunctionByName(callback, [payload], this);
                        calledCallback.push(callback);
                    });
                }

                // call registered callback from data-callback
                if (this.aitomCallback !== null) {
                    this.aitomCallback.apply(null, [payload]);
                    this.aitomCallback = null; // reset callback for further requests
                }
            },
            complete: function () {
                // clear for new requests
                this.currentXhr = null;
            },
            error: function (xhr, status, error) {

            }
        }, {
            // this is shared context of all above callbacks
            aitomCallback: null,
            currentXhr: null // actually running xhr request
        });
    }

}

const AitomAjax = new _AitomAjax();
export default AitomAjax;